#!/bin/env python
# -*- coding: UTF-8 -*-
## \namespace cga_act
# \brief Conjunto de clases bases de la aplicacion CGA Actualizacion

import pango
import pygtk
pygtk.require('2.0')
import gtk
import os
from threading import Thread
import thread
import time

##\class cga_Act_Base
# \brief Clase base para la creacion de la aplicacion
# \version 0.0.1
# \date 28/04/2008
class cga_Act_Base:
	##\fn __init__(self)
	# \brief Constructor de la clase. Inicializa los parametros basicos de la clase
	# \param[in] self Instancia de la clase que realiza la llamada
	def __init__(self):
		# Ventana principal de la aplicacion
		self.PATH = '/usr/local/cga-act/'
		self.crear_interfaz()

	##\fn crear_interfaz(self)
	# \brief Metodo encargado de crear los elementos de la interfaz de usuario
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def crear_interfaz(self):
		# ventana principal
		self.window = gtk.Window (gtk.WINDOW_TOPLEVEL)
		self.window.set_border_width(10)
		self.window.set_icon_from_file('%simg/cga_act.png' % self.PATH)
		self.window.set_title('CGA Actualización')
		#self.window.set_resizable(False)
		self.window.set_size_request(500,500)
		self.window.connect("delete_event", self.delete_event)
		self.window.connect("destroy", self.destroy)

		# contenedores
		self.button_box = gtk.HBox(True, 10)
		self.app_box = gtk.VBox(False, 10)
		self.app_box.pack_start(self.button_box, False, False, 10)
		
		# botones de comprobacion y actualizacion
		self.button_comp = gtk.Button("Comprobar actualizaciones")
		self.button_comp.connect("clicked", self.comprobar_actualizacion)
		self.button_comp.set_size_request(200, 35)
		self.button_box.pack_start(self.button_comp, False, False, 0)
		self.button_forz = gtk.Button("Forzar actualización")
		self.button_forz.connect("clicked", self.forzar_actualizacion)
		self.button_forz.set_size_request(200, 35)
		self.button_forz.set_sensitive(False)
		self.button_box.pack_start(self.button_forz, False, False, 0)

		# cuadro de salida de texto
		self.text_cont_desp = gtk.ScrolledWindow()
		self.text_cont_desp.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.text_cont_desp.set_shadow_type(gtk.SHADOW_IN)
		self.text_mens = gtk.TextView(None)
		self.text_mens.set_pixels_above_lines(1)
		self.text_mens.set_editable(False)
		self.text_buffer = self.text_mens.get_buffer()
		self.text_cont_desp.add(self.text_mens)
		self.app_box.pack_end(self.text_cont_desp, True, True, 14)
		self.tag_text_correcto = self.text_buffer.create_tag("correcto")
		self.tag_text_correcto.set_property('foreground', 'green')
		self.tag_text_correcto.set_property('weight', pango.WEIGHT_BOLD)
		self.tag_text_error = self.text_buffer.create_tag("error")
		self.tag_text_error.set_property('weight', pango.WEIGHT_BOLD)
		self.tag_text_error.set_property('foreground', 'red')
		self.tag_text_cabecera = self.text_buffer.create_tag("cabecera")
		self.tag_text_cabecera.set_property('weight', pango.WEIGHT_BOLD)
		self.tag_text_cabecera.set_property('foreground', 'blue')

		# incluir elementos en la ventana
		self.window.add(self.app_box)

	##\fn actualiza_salida_comp(self)
	# \brief Funcion para actualizar la salida de texto cuando se realiza la comprobacion de actualizaciones
	# \param[in] self Instanacia de la clase que realiza la llamada al metodo
	def actualiza_salida_comp(self):
		cabecera = time.strftime("[%m/%d/%y %H:%M:%S] - Iniciando comprobación", time.localtime())
		self.text_buffer.insert(self.text_buffer.get_end_iter(), cabecera)
		self.desplazar_scroll_final()
		iter_aux = self.text_buffer.get_end_iter()
		iter_aux.backward_visible_line()
		self.text_buffer.apply_tag(self.tag_text_cabecera, iter_aux, self.text_buffer.get_end_iter())
		self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n")
		for linea in self.stdouterror_comp.readlines():
			self.text_buffer.insert(self.text_buffer.get_end_iter(), linea)
			self.desplazar_scroll_final()
		iter_aux = self.text_buffer.get_end_iter()
		iter_aux.backward_visible_line()
		self.text_buffer.delete(iter_aux, self.text_buffer.get_end_iter())
		self.desplazar_scroll_final()
		if linea == "0\n":
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n")
			self.desplazar_scroll_final()
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "Este equipo se encuentra actualizado correctamente.")
			self.desplazar_scroll_final()
			iter_aux = self.text_buffer.get_end_iter()
			iter_aux.backward_visible_line()
			self.text_buffer.apply_tag(self.tag_text_correcto, iter_aux, self.text_buffer.get_end_iter())
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n\n")
			self.desplazar_scroll_final()
		else:
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n")
			self.desplazar_scroll_final()
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "Se han producido errores en la comprobación.")
			self.desplazar_scroll_final()
			iter_aux = self.text_buffer.get_end_iter()
			iter_aux.backward_visible_line()
			self.text_buffer.apply_tag(self.tag_text_error, iter_aux, self.text_buffer.get_end_iter())
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n")
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "Pruebe con la opción Forzar actualización para intentar")
			self.desplazar_scroll_final()
			iter_aux = self.text_buffer.get_end_iter()
			iter_aux.backward_visible_line()
			self.text_buffer.apply_tag(self.tag_text_error, iter_aux, self.text_buffer.get_end_iter())
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n")
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "solucionar el problema.")
			self.desplazar_scroll_final()
			iter_aux = self.text_buffer.get_end_iter()
			iter_aux.backward_visible_line()
			self.text_buffer.apply_tag(self.tag_text_error, iter_aux, self.text_buffer.get_end_iter())
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n\n")
			self.button_forz.set_sensitive(True)
			self.desplazar_scroll_final()

	##\fn actualiza_salida_forz(self)
	# \brief Funcion para actualizar la salida de texto cuando se realiza el forzado de actualizaciones
	# \param[in] self Instanacia de la clase que realiza la llamada al metodo
	def actualiza_salida_forz(self):
		cabecera = time.strftime("[%m/%d/%y %H:%M:%S] - Iniciando forzado", time.localtime())
		self.text_buffer.insert(self.text_buffer.get_end_iter(), cabecera)
		self.desplazar_scroll_final()
		iter_aux = self.text_buffer.get_end_iter()
		iter_aux.backward_visible_line()
		self.text_buffer.apply_tag(self.tag_text_cabecera, iter_aux, self.text_buffer.get_end_iter())
		self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n")
		for linea in self.stdouterror_forz.readlines():
			self.text_buffer.insert(self.text_buffer.get_end_iter(), linea)
			self.desplazar_scroll_final()
		iter_aux = self.text_buffer.get_end_iter()
		iter_aux.backward_visible_line()
		self.text_buffer.delete(iter_aux, self.text_buffer.get_end_iter())
		self.desplazar_scroll_final()
		if linea == "0\n":
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n")
			self.desplazar_scroll_final()
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "El problema encontrado ha sido solucionado y su equipo")
			self.desplazar_scroll_final()
			iter_aux = self.text_buffer.get_end_iter()
			iter_aux.backward_visible_line()
			self.text_buffer.apply_tag(self.tag_text_correcto, iter_aux, self.text_buffer.get_end_iter())
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n")
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "se encuentra actualizado correctamente.")
			self.desplazar_scroll_final()
			iter_aux = self.text_buffer.get_end_iter()
			iter_aux.backward_visible_line()
			self.text_buffer.apply_tag(self.tag_text_correcto, iter_aux, self.text_buffer.get_end_iter())
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n\n")
			self.desplazar_scroll_final()
		else:
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n")
			self.desplazar_scroll_final()
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "El problema encontrado no ha podido ser solucionado.")
			self.desplazar_scroll_final()
			iter_aux = self.text_buffer.get_end_iter()
			iter_aux.backward_visible_line()
			self.text_buffer.apply_tag(self.tag_text_error, iter_aux, self.text_buffer.get_end_iter())
			self.text_buffer.insert(self.text_buffer.get_end_iter(), "\n\n")
			self.desplazar_scroll_final()
		self.button_forz.set_sensitive(False)
	
	##\fn comprobar_actualizacion(self, widget, data = None)
	# \brief Funcion que comprueba el estado de las actualizaciones y si es posible actualiza el sistema
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados a la funcion
	def comprobar_actualizacion(self, widget, data = None):
		self.stdin_comp, self.stdouterror_comp = os.popen4('apt-get update;apt-get -u -y -f dist-upgrade;echo $?')
		self.hilo_actualizar_comp = thread.start_new_thread(self.actualiza_salida_comp,())

	##\fn forzar_actualizacion(self, widget, data = None)
	# \brief Funcion que intenta reparar el sistema de paquetes del sistema y posteriormente actualiza el equipo
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados a la funcion
	def forzar_actualizacion(self, widget, data = None):
		self.stdin_forz, self.stdouterror_forz = os.popen4('dpkg --configure -a;apt-get update;apt-get -f -y install;apt-get -u -y -f dist-upgrade;echo $?')
		self.hilo_actualizar_forz = thread.start_new_thread(self.actualiza_salida_forz,())

	##\fn desplazar_scroll_final(self)
	# \brief Funcion que desplaza el scroll del cuadro de texto hasta la ultima linea
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def desplazar_scroll_final(self):
		endmark = self.text_buffer.create_mark(None, self.text_buffer.get_end_iter())
		self.text_mens.scroll_mark_onscreen(endmark)
		

	##\fn delete_event(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la aplicacion
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados a la funcion
	def delete_event(self, widget, data = None):
		gtk.main_quit()
	
	##\fn destroy(self, widget, data = None)
	# \brief Funcion para terminar la ejecucion de la aplicacion
	# \param[in] self Instancia de la clase que realiza la llamada
	# \param[in] widget Widget asociado al evento
	# \param[in] data Datos pasados a la funcion
	def destroy(self, widget, data = None):
		gtk.main_quit()

	##\fn show(self)
	# \brief Funcion que dibuja la ventana y todos sus elementos por pantalla
	# \param[in] self Instancia de la clase que realiza la llamada al metodo
	def show(self):
		self.window.show_all()
